<?php

namespace App\Http\Controllers\Api;

use App\Models\Subject;
use Illuminate\Http\JsonResponse;
use Throwable;

class SubjectApiController extends BaseAPIController
{
    /**
     * Get Subjects
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        $output = self::baseResponse();

        try {
            $output['message'] = "List of Subjects";
            $output['data'] = Subject::orderBy('course_year')->orderBy('name')->get();
        } catch (Throwable $error) {
            $output['error'] = true;
            $output['status'] = 400;
            $output['message'] = $error->getMessage();
            $output['errors'][] = $error->getMessage();
        }

        return response()->json($output)->setStatusCode($output['status']);
    }

    /**
     * Subject Info
     * @param Subject $subject
     * @return JsonResponse
     */
    public function show(Subject $subject): JsonResponse
    {
        $output = self::baseResponse();

        try {
            $output['message'] = "Subject info";
            $output['data'] = $subject;
        } catch (Throwable $error) {
            $output['error'] = true;
            $output['status'] = 400;
            $output['message'] = $error->getMessage();
            $output['errors'][] = $error->getMessage();
        }

        return response()->json($output)->setStatusCode($output['status']);
    }

    /**
     * Subject Info with Score Average
     * @param Subject $subject
     * @return JsonResponse
     */
    public function showWithScoreAverage(Subject $subject): JsonResponse
    {
        $output = self::baseResponse();

        try {
            $subjectExtraInfo = Subject::withAvg('users AS score_average', 'users_has_subjects.score')
                ->find($subject->id);

            $output['message'] = "Subject info with score average";
            $output['data'] = $subjectExtraInfo;
        } catch (Throwable $error) {
            $output['error'] = true;
            $output['status'] = 400;
            $output['message'] = $error->getMessage();
            $output['errors'][] = $error->getMessage();
        }

        return response()->json($output)->setStatusCode($output['status']);
    }
}
