<?php

namespace App\Http\Controllers\Api;

use App\Models\User;
use Illuminate\Http\JsonResponse;
use Throwable;

class UserApiController extends BaseAPIController
{
    /**
     * Get Users
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        $output = self::baseResponse();

        try {
            $output['message'] = "List of Users";
            $output['data'] = User::orderBy('name')->orderBy('lastname')->get();
        } catch (Throwable $error) {
            $output['error'] = true;
            $output['status'] = 400;
            $output['message'] = $error->getMessage();
            $output['errors'][] = $error->getMessage();
        }

        return response()->json($output)->setStatusCode($output['status']);
    }

    /**
     * User Info
     * @param User $user
     * @return JsonResponse
     */
    public function show(User $user): JsonResponse
    {
        $output = self::baseResponse();

        try {
            $output['message'] = "User info";
            $output['data'] = $user;
        } catch (Throwable $error) {
            $output['error'] = true;
            $output['status'] = 400;
            $output['message'] = $error->getMessage();
            $output['errors'][] = $error->getMessage();
        }

        return response()->json($output)->setStatusCode($output['status']);
    }

    /**
     * User Info With Scores and subjects average
     * @param User $user
     * @return JsonResponse
     */
    public function showWithScores(User $user): JsonResponse
    {
        $output = self::baseResponse();

        try {
            $userExtraInfo = User::with(['subjects'])
                ->withAvg('subjects AS subjects_score_average', 'users_has_subjects.score')
                ->find($user->id);

            $output['message'] = "User info with scores and subjects score average";
            $output['data'] = $userExtraInfo;
        } catch (Throwable $error) {
            $output['error'] = true;
            $output['status'] = 400;
            $output['message'] = $error->getMessage();
            $output['errors'][] = $error->getMessage();
        }

        return response()->json($output)->setStatusCode($output['status']);
    }
}
