<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'password',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    protected $appends = [
        'fullname',
        'age',
    ];

    /**
     * Get Full Name
     * @return string Full Name
     */
    public function getFullnameAttribute(): string
    {
        return $this->attributes['name'] . ' ' . $this->attributes['lastname'];
    }

    /**
     * Get Age
     * @return int Age
     */
    public function getAgeAttribute(): int
    {
        $birthdate = $this->attributes['birthdate'];
        $today = Carbon::now();

        $years = $today->diff($birthdate);

        return $years->y;
    }

    /**
     * @return BelongsToMany
     */
    public function subjects(): BelongsToMany
    {
        return $this->belongsToMany(Subject::class, 'users_has_subjects')
            ->withPivot(['score']);
    }
}
